/********************************************
  
  orbit7gui.c :  GTK front end for Orbit7

  By Ben Collins-Sussman <sussman@red-bean.com>

*******************************************/



/* 
 * This file is part of orbit7, a gravity simulator.
 * Copyright (C) 1998, 1999  Ben Sussman and Mike Sussman 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include "orbit7.h"

int win_width = 600, win_height = 600;

/* ------------- Global GTK/GDK Widgets ----------------------------*/

GtkWidget *mainwindow, *drawingarea, *vbox, *hbox;
GtkWidget *vbox1, *dial1, *label1;  // sun mass
GtkWidget *vbox2, *dial2, *label2;  // planet mass
GtkWidget *togglebutton, *togglelabel; // spring_p?
GtkWidget *quitbutton, *resetbutton;
static GdkPixmap *pixmap = NULL;
GtkAdjustment *adjustment1, *adjustment2;


/*------------- Event-Handling Callback Routines -------------------*/

/* Callback to close our window */
gint handle_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  /* quit the whole program */
  gtk_main_quit();
}

/* If the window is "configured" (resized), then we need to create 
   a new backing pixmap and restart the whole planetary system. */

static gint
configure_event (GtkWidget *widget, GdkEventConfigure *event)
{
  if (pixmap)
    gdk_pixmap_unref(pixmap);
  
  pixmap = gdk_pixmap_new(widget->window,
			  widget->allocation.width,
			  widget->allocation.height,
			  -1);
  gdk_draw_rectangle (pixmap,
		      widget->style->black_gc,
		      TRUE,
		      0, 0,
		      widget->allocation.width,
		      widget->allocation.height);

  win_width = widget->allocation.width,
  win_height = widget->allocation.height;
  
  generate_system();
  
  return TRUE;
}


/* When we receive an exposure event, copy the appropriate rectangular
   portion of our pixmap to the screen. */
static gint
expose_event (GtkWidget *widget, GdkEventExpose *event)
{
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		  pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
  return FALSE;
}

/*  Deal with gravity/springforce toggling */

void toggle_button_callback(GtkWidget *widget, gpointer data)
{
  if (GTK_TOGGLE_BUTTON(widget)->active)  {
    /* If control reaches here, the toggle button is down */
    gtk_label_set(GTK_LABEL(togglelabel), "Using Springforce");
    spring_p = 1;
  } 
  else {
    /* If control reaches here, the toggle button is up */
    gtk_label_set(GTK_LABEL(togglelabel), "Using Gravity");
    spring_p = 0;
  }
}

/* Reset button resets the system */

void reset_button_callback(GtkWidget *widget, gpointer data)
{
  GdkRectangle rect;

  if (pixmap)
    gdk_pixmap_unref(pixmap);
  
  pixmap = gdk_pixmap_new(widget->window,
			  win_width,
			  win_height,
			  -1);
  gdk_draw_rectangle (pixmap,
		      widget->style->black_gc,
		      TRUE,
		      0, 0,
		      win_width,
		      win_height);  

  rect.x = 0; rect.y = 0; rect.width = win_width; rect.height = win_height;
  gtk_widget_draw(GTK_WIDGET(drawingarea), &rect);

  generate_system();  
}


/* When we move the dials, change things... */

void value_changed1(GtkAdjustment *adjustment, GtkWidget *label)
{
  char buffer[40];

  sun_mass = (double) -((double)adjustment->value  *  1000);
  position[4] = sun_mass;

  sprintf(buffer,"Sun Mass: %5.0f", -sun_mass);
  gtk_label_set(GTK_LABEL (label), buffer);

}

void value_changed2(GtkAdjustment *adjustment, GtkWidget *label)
{
  char buffer[40];
  int count;

  planet_mass = (double) adjustment->value;

  for (count = 1; count < NUM_ELEMENTS; count++)
    position[5*count + 4] = -planet_mass;

  sprintf(buffer,"Planet Mass: %5.2f", planet_mass);
  gtk_label_set(GTK_LABEL (label), buffer);
}


/* -------------- Draw a planet ----------------------------------*/


/* The only routine called by the mathematics code in orbit7.c */
void my_putpixel(int i, int j, int color)
{
  GdkRectangle rect;

  if ((i > win_width) || (j > win_width) || (i < 0) || (j < 0))
    return;

  // Define a 2x2 rectangle as our "pixel" to put
  rect.x = i;
  rect.y = j;
  rect.width = 2;
  rect.height = 2;

  // Draw either a black or white rectangle to our backing pixmap
  if (color == 0) {
    gdk_draw_rectangle(pixmap, drawingarea->style->black_gc, 
		       TRUE,
		       rect.x, rect.y, rect.width, rect.height);
  }
  else {
    gdk_draw_rectangle(pixmap, drawingarea->style->white_gc, 
		       TRUE,
		       rect.x, rect.y, rect.width, rect.height);   
  }

  // And create a "exposure" event for the rectangle, which will cause
  // the correct portion of the pixmap to be copied to the screen
  gtk_widget_draw(GTK_WIDGET(drawingarea), &rect);

}


/* ------------------- GTK Setup Stuff ---------------------------*/


/*  Create all our GTK main-window widgets here */

void layout_widgets()
{
  char buf[40];
  
  // Our main application window
  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(mainwindow), "gorbit7");
  gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event",
		     GTK_SIGNAL_FUNC(handle_delete_event), NULL);
  // Uncomment this once everyone is using GTK 1.2:
  // gtk_container_set_border_width(GTK_CONTAINER(mainwindow), 10);

  // Uber-Vbox to hold widgets
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(mainwindow), vbox);
  gtk_widget_show(vbox);

  // Our drawing area (for planets)
  drawingarea = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea), win_width, win_height);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
		     (GtkSignalFunc) expose_event, NULL);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "configure_event",
		     (GtkSignalFunc) configure_event, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), drawingarea, TRUE, TRUE, 0);
  gtk_widget_show(drawingarea);
  
  // Hbox goes below drawing area.
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);

  // Our dials!
  // Sun Mass:
  vbox1 = gtk_vbox_new(FALSE, 0);
  adjustment1 = GTK_ADJUSTMENT(gtk_adjustment_new
			       (-(sun_mass/1000), -1, 160, .1, 1, 0));
  dial1 = gtk_dial_new(adjustment1);
  gtk_dial_set_update_policy (GTK_DIAL(dial1), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(vbox1), dial1, FALSE, FALSE, 0);
  gtk_widget_show(dial1);
  sprintf(buf, "Sun Mass: %5.0f", -sun_mass);
  label1 = gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(vbox1), label1, TRUE, TRUE, 0);
  gtk_widget_show(label1);
  gtk_signal_connect (GTK_OBJECT(adjustment1), "value_changed",
		      GTK_SIGNAL_FUNC (value_changed1), label1);
  gtk_widget_show(vbox1);
  gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 10);
  // Planet Mass:
  vbox2 = gtk_vbox_new(FALSE, 0);
  adjustment2 = GTK_ADJUSTMENT(gtk_adjustment_new
			       (planet_mass, 0, 10, .01, .1, 0));
  dial2 = gtk_dial_new(adjustment2);
  gtk_dial_set_update_policy (GTK_DIAL(dial2), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(vbox2), dial2, FALSE, FALSE, 0);
  gtk_widget_show(dial2);
  sprintf(buf, "Planet Mass: %1.2f", planet_mass);
  label2 = gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(vbox2), label2, TRUE, TRUE, 0);
  gtk_widget_show(label2);
  gtk_signal_connect (GTK_OBJECT(adjustment2), "value_changed",
		      GTK_SIGNAL_FUNC (value_changed2), label2);
  gtk_widget_show(vbox2);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 10);

  // Togglebutton for gravity/springforce
  togglebutton = gtk_toggle_button_new();
  gtk_box_pack_start(GTK_BOX(hbox), togglebutton, TRUE, TRUE, 10);
  gtk_signal_connect (GTK_OBJECT(togglebutton), "toggled",
		      GTK_SIGNAL_FUNC (toggle_button_callback), NULL);
  togglelabel = gtk_label_new("Using Gravity");
  gtk_container_add(GTK_CONTAINER(togglebutton), togglelabel);
  gtk_widget_show(togglelabel);
  gtk_widget_show(togglebutton);

  // Reset button
  resetbutton = gtk_button_new_with_label("Reset");
  gtk_box_pack_start(GTK_BOX(hbox), resetbutton, TRUE, TRUE, 10);
  gtk_signal_connect(GTK_OBJECT(resetbutton), "clicked",
		     GTK_SIGNAL_FUNC(reset_button_callback), NULL);
  gtk_widget_show(resetbutton);

  // Quit button
  quitbutton = gtk_button_new_with_label("Stop the universe,\nI wanna get off.");
  gtk_box_pack_start(GTK_BOX(hbox), quitbutton, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(quitbutton), "clicked",
		     GTK_SIGNAL_FUNC(handle_delete_event), NULL);
  gtk_widget_show(quitbutton);


  // display top-level window
  gtk_widget_show(mainwindow);
}



/*  Entry point for GTK initialization, called by main() */

void start_X11_gui(int argc, char *argv[])
{
  /* allow gtk to look for standardized command-line args */
  gtk_init(&argc, &argv);
  
  /* time to make the widgets */
  layout_widgets();

  /* tell gtk to move the planets when event loop is idle! */
  gtk_idle_add(do_work, 0);
  
  /* start the event loop */
  gtk_main();
}









